@@ -1,4 +1,5 @@
-# Copyright 2019 Vitaliy Zakaznikov (TestFlows Test Framework http://testflows.com)
+# Copyright 2019 Katteli Inc.
+# TestFlows Test Framework (http://testflows.com)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,8 +18,7 @@ import importlib
 import threading
 
 from .exceptions import ResultException
-from .serialize import dumps
-from .message import Message
+from .message import Message, dumps
 from .objects import OK, Fail, Error, Skip, Null
 from .objects import XOK, XFail, XError, XNull
 from .objects import Value, Metric, Ticket
@@ -151,10 +151,10 @@ def message(message, test=None):
         test = current()
     test.io.output.message(Message.NONE, dumps(str(message)))
 
-def exception(test=None):
+def exception(exc_type=None, exc_value=None, exc_traceback=None, test=None):
     if test is None:
         test = current()
-    test.io.output.exception()
+    test.io.output.exception(exc_type, exc_value, exc_traceback)
 
 def ok(message=None, test=None):
     if test is None:
@@ -228,4 +228,4 @@ def getsattr(obj, name, *default):
     """
     value = getattr(obj, name, *default)
     setattr(obj, name, value)
-    return value
\ No newline at end of file
+    return value
