@@ -1,4 +1,5 @@
-# Copyright 2019 Vitaliy Zakaznikov (TestFlows Test Framework http://testflows.com)
+# Copyright 2019 Katteli Inc.
+# TestFlows Test Framework (http://testflows.com)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,6 +42,11 @@ from .cli.text import danger, warning
 from .exceptions import exception as get_exception
 from .filters import the
 
+try:
+    import testflows.database as database_module
+except:
+    database_module = None
+
 class xfails(dict):
     """xfails container.
 
@@ -180,17 +186,15 @@ class TestBase(object):
     tags = set()
     attributes = []
     requirements = []
-    users = []
-    tickets = []
     examples = None
     name = None
     description = None
     node = None
-    map = None
+    map = []
     flags = Flags()
     name_sep = "."
     type = TestType.Test
-    subtype = TestSubType.Empty
+    subtype = None
 
     @classmethod
     def argparser(cls):
@@ -241,13 +245,15 @@ class TestBase(object):
             help="disable terminal color highlighting", default=False)
         parser.add_argument("--id", metavar="id", dest="_id", type=str, help="custom test id")
         parser.add_argument("-o", "--output", dest="_output", metavar="format", type=str,
-            choices=["slick", "nice", "quiet", "short", "dots", "raw", "silent"], default="nice",
-            help="""stdout output format, choices are: ['slick','nice','short','dots','quiet','raw','silent'],
+            choices=["slick", "nice", "quiet", "short", "dots", "raw"], default="nice",
+            help="""stdout output format, choices are: ['slick','nice','short','dots','quiet','raw'],
                 default: 'nice'""")
         parser.add_argument("-l", "--log", dest="_log", metavar="file", type=str,
             help="path to the log file where test output will be stored, default: uses temporary log file")
         parser.add_argument("--show-skipped", dest="_show_skipped", action="store_true",
             help="show skipped tests, default: False", default=False)
+        if database_module:
+            database_module.argparser(parser)
         return parser
 
     def parse_cli_args(self, xflags=None, only=None, skip=None, start=None, end=None,
@@ -295,6 +301,9 @@ class TestBase(object):
 
             settings.output_format = args.pop("_output")
 
+            if args.get("_database"):
+                settings.database = args.pop("_database")
+
             if args.get("_show_skipped"):
                 settings.show_skipped = True
                 args.pop("_show_skipped")
@@ -338,7 +347,7 @@ class TestBase(object):
                 args.pop("_end")
 
             if args.get("_tags"):
-                tags = [value for value in args.pop("_tags")]
+                tags = {value for value in args.pop("_tags")}
 
             if args.get("_attrs"):
                 attributes = [Attribute(item.key, item.value) for item in args.pop("_attrs")]
@@ -356,7 +365,7 @@ class TestBase(object):
 
     def __init__(self, name=None, flags=None, cflags=None, type=None, subtype=None,
                  uid=None, tags=None, attributes=None, requirements=None,
-                 users=None, tickets=None, examples=None, description=None, parent=None,
+                 examples=None, description=None, parent=None,
                  xfails=None, xflags=None, only=None, skip=None,
                  start=None, end=None, args=None, id=None, node=None, map=None, context=None,
                  _frame=None, _run=True):
@@ -384,21 +393,19 @@ class TestBase(object):
         self.parent = parent
         self.id = get(id, [settings.test_id])
         self.node = get(node, self.node)
-        self.map = get(map, self.map)
+        self.map = get(map, list(self.map))
         self.type = get(type, self.type)
         self.subtype = get(subtype, self.subtype)
         self.context = get(context, current().context if current() and self.type < TestType.Test else (Context(current().context if current() else None)))
         self.tags = tags
-        self.requirements = get(requirements, self.requirements)
-        self.attributes = get(attributes, self.attributes)
-        self.users = get(users, self.users)
-        self.tickets = get(tickets, self.tickets)
+        self.requirements = get(requirements, list(self.requirements))
+        self.attributes = get(attributes, list(self.attributes))
         self.description = get(description, self.description)
         self.examples = get(examples, get(self.examples, ExamplesTable()))
         self.args = get(args, {})
         self.args.update({k:v for k, v in cli_args.items() if not k.startswith("_")})
         self._process_args()
-        self.result = Null(self.name)
+        self.result = Null(test=self.name)
         if flags is not None:
             self.flags = Flags(flags)
         self.cflags = Flags(cflags) | (self.flags & CFLAGS)
@@ -468,7 +475,7 @@ class TestBase(object):
         current(self)
 
         if self.flags & SKIP:
-            raise ResultException(Skip(self.name, "skip flag set"))
+            raise Skip("skip flag set", test=self.name)
         else:
             if top() is self:
                 self._init = init()
@@ -476,44 +483,47 @@ class TestBase(object):
                 self.run(**{name: arg.value for name, arg in self.args.items()})
             return self
 
-    def __exit__(self, exception_type, exception_value, exception_traceback):
+    def __exit__(self, exc_type, exc_value, exc_traceback):
         if top() is self and not self._init:
             return False
 
-        def process_exception(exception_type, exception_value, exception_traceback):
-            if isinstance(exception_value, ResultException):
-                self.result = self.result(exception_value.result)
-            elif isinstance(exception_value, AssertionError):
-                exception(test=self)
-                self.result = self.result(Fail(self.name, str(exception_value)))
+        def process_exception(exc_type, exc_value, exc_traceback):
+            if isinstance(exc_value, ResultException):
+                self.result = self.result(exc_value)
+            elif isinstance(exc_value, AssertionError):
+                exception(exc_type, exc_value, exc_traceback, test=self)
+                self.result = self.result(Fail(str(exc_value), test=self.name))
             else:
-                exception(test=self)
-                result = Error(self.name,
-                    "unexpected %s: %s" % (exception_type.__name__, str(exception_value)))
+                exception(exc_type, exc_value, exc_traceback, test=self)
+                result = Error("unexpected %s: %s" % (exc_type.__name__, str(exc_value)), test=self.name)
                 self.result = self.result(result)
-                if isinstance(exception_value, KeyboardInterrupt):
-                    raise ResultException(result)
+                if isinstance(exc_value, KeyboardInterrupt):
+                    raise result
 
         try:
-            if exception_value:
-                process_exception(exception_type, exception_value, exception_traceback)
+            if exc_value is not None:
+                process_exception(exc_type, exc_value, exc_traceback)
             else:
                 if isinstance(self.result, Null):
-                    self.result = self.result(OK(self.name))
+                    self.result = self.result(OK(test=self.name))
         finally:
             try:
                 if self.type >= TestType.Test:
                     if self.context._cleanups:
                         with Finally("I clean up"):
                             cleanup_exc_type, cleanup_exc_value, cleanup_exc_traceback = self.context._cleanup()
-                        if not exception_value and cleanup_exc_value:
+                        if not exc_value and cleanup_exc_value:
                             process_exception(cleanup_exc_type, cleanup_exc_value, cleanup_exc_traceback)
             finally:
                 current(self.caller_test)
                 previous(self)
                 self._apply_xfails()
                 self.io.output.result(self.result)
-                self.io.close()
+                if top() is self:
+                    self.io.output.stop()
+                    self.io.close(final=True)
+                else:
+                    self.io.close()
 
             if self.flags & PAUSE_AFTER:
                 pause()
@@ -699,7 +709,7 @@ class _test(object):
         :param kwargs: test's kwargs
         """
         type = kwargs.pop("type", TestType.Test)
-        subtype = kwargs.pop("subtype", TestSubType.Empty)
+        subtype = kwargs.pop("subtype", None)
 
         parent_type = parent.type
 
@@ -757,7 +767,7 @@ class _test(object):
                 __kwargs.pop("name", None)
                 __kwargs.pop("parent", None)
                 __kwargs["type"] = TestType.Iteration
-                __kwargs["subtype"] = TestSubType.Empty
+                __kwargs["subtype"] = None
                 for i in range(self._repeat):
                     with Iteration(name=f"{i}", tags=self._tags, **__kwargs, parent_type=self.test.type):
                         exec(code, frame.f_globals, frame.f_locals)
@@ -787,13 +797,13 @@ class _test(object):
                 sys.exit(1)
 
             if isinstance(self.test.result, Fail):
-                result = Fail(self.parent.name, self.test.result.message)
+                result = Fail(test=self.parent.name, message=self.test.result.message)
             else:
                 # convert Null into an Error
-                result = Error(self.parent.name, self.test.result.message)
+                result = Error(test=self.parent.name, message=self.test.result.message)
 
             if TE not in self.test.flags:
-                raise ResultException(result)
+                raise result
             else:
                 with self.parent.lock:
                     if isinstance(self.parent.result, Error):
@@ -870,6 +880,12 @@ class Steps(ExitStack):
         self.values.append(step)
         return step
 
+@contextmanager
+def Cleanup(callback, *args, **kwargs):
+    test = kwargs.pop("_test", current())
+    test.context.cleanup(callback, *args, **kwargs)
+    yield
+
 @contextmanager
 def Background(name, **kwargs):
     with _background(name, **kwargs) as bg:
@@ -968,7 +984,7 @@ class _testdecorator(object):
                 __kwargs = dict(_kwargs)
                 __kwargs.pop("name")
                 __kwargs["type"] = TestType.Iteration
-                __kwargs["subtype"] = TestSubType.Empty
+                __kwargs["subtype"] = None
                 for i in range(_repeat):
                     with Iteration(name=f"{i}", parent_type=parent_test.type, **__kwargs) as test:
                         self.func(test, **{name: arg.value for name, arg in test.args.items()})
@@ -1065,22 +1081,6 @@ class Uid(object):
         func.uid = self.uid
         return func
 
-class Users(object):
-    def __init__(self, *users):
-        self.users = users
-
-    def __call__(self, func):
-        func.users = self.users
-        return func
-
-class Tickets(object):
-    def __init__(self, *tickets):
-        self.tickets = tickets
-
-    def __call__(self, func):
-        func.tickets = self.tickets
-        return func
-
 def run(comment=None, test=None, **kwargs):
     """Run a test.
 
@@ -1108,7 +1108,7 @@ def run(comment=None, test=None, **kwargs):
                                _frame=_frame, _run=False) as parent_test:
             _kwargs = dict(kwargs)
             _kwargs["type"] = TestType.Iteration
-            _kwargs["subtype"] = TestSubType.Empty
+            _kwargs["subtype"] = None
             for i in range(_repeat):
                 with Iteration(test=test, name=f"{i}", **kwargs, _frame=_frame, parent_type=parent_test.type):
                     pass
