@@ -1,4 +1,5 @@
-# Copyright 2019 Vitaliy Zakaznikov (TestFlows Test Framework http://testflows.com)
+# Copyright 2019 Katteli Inc.
+# TestFlows Test Framework (http://testflows.com)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,7 +12,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-class TestType(object):
+from .utils.enum import IntEnum
+
+class TestType(IntEnum):
     """Test type."""
     Module = 40
     Suite = 30
@@ -19,7 +22,7 @@ class TestType(object):
     Iteration = 15
     Step = 10
 
-class TestSubType(object):
+class TestSubType(IntEnum):
     """Test behaviour subtype."""
     Feature = 60
     Scenario = 50
@@ -31,5 +34,3 @@ class TestSubType(object):
     But = 7
     By = 6
     Finally = 5
-    Empty = 0
-
